
static char rcsid[] = "@(#)$Id: addr_util.c,v 1.2 1998/10/20 19:58:15 hurtta Exp $";

/******************************************************************************
 *  The Elm (ME+) Mail System  -  $Revision: 1.2 $   $State: Exp $
 *
 *  Modified by: Kari Hurtta <hurtta+elm@ozone.FMI.FI>
 ******************************************************************************
 *  The Elm Mail System 
 *
 * 			Copyright (c) 1988-1992 USENET Community Trust
 * 			Copyright (c) 1986,1987 Dave Taylor
 *****************************************************************************/

/** This file contains addressing utilities 

**/

#include "headers.h"
#include "s_elm.h"
#include "me.h"

translate_return(addr, ret_addr)
char *addr, *ret_addr;
{
	/** Return ret_addr to be the same as addr, but with the login 
            of the person sending the message replaced by '%s' for 
            future processing... 
	    Fixed to make "%xx" "%%xx" (dumb 'C' system!) 
	**/

	register int loc, loc2, iindex = 0;
	register char *remaining_addr;
	
/*
 *	check for RFC-822 source route: format @site:usr@site
 *	if found, skip to after the first : and then retry.
 *	source routes can be stacked
 */
	remaining_addr = addr;
	while (*remaining_addr == '@') {
	  loc = qchloc(remaining_addr, ':');
	  if (loc == -1)
	    break;

	  remaining_addr += loc + 1;
	}

	loc2 = qchloc(remaining_addr,'@');
	loc = qchloc(remaining_addr, '%');
	if ((loc < loc2) && (loc != -1))
	  loc2 = loc;

	if (loc2 != -1) {	/* ARPA address. */
	  /* algorithm is to get to '@' sign and move backwards until
	     we've hit the beginning of the word or another metachar.
	  */
	  for (loc = loc2 - 1; loc > -1 && remaining_addr[loc] != '!'; loc--)
	     ;
	}
	else {			/* usenet address */
	  /* simple algorithm - find last '!' */

	  loc2 = strlen(remaining_addr);	/* need it anyway! */

	  for (loc = loc2; loc > -1 && remaining_addr[loc] != '!'; loc--)
	      ;
	}
	
	/** now copy up to 'loc' into destination... **/

	while (iindex <= loc) {
	  ret_addr[iindex] = remaining_addr[iindex];
	  iindex++;
	}

	/** now append the '%s'... **/

	ret_addr[iindex++] = '%';
	ret_addr[iindex++] = 's';

	/*
	 *  and, finally, if anything left, add that
	 * however, just pick up the address part, we do
	 * not want any comments.  Thus stop copying at
	 * the first blank character.
	 */

	if ((loc = qchloc(remaining_addr,' ')) == -1)
	  loc = strlen(addr);
	while (loc2 < loc) {
	  ret_addr[iindex++] = remaining_addr[loc2++];
	  if (remaining_addr[loc2-1] == '%')	/* tweak for "printf" */
	    ret_addr[iindex++] = '%';
	}
	
	ret_addr[iindex] = '\0';
}

int build_address(to, full_to, size_to, size_full_to)
     char *to, *full_to;
     int size_to, size_full_to;
{
	/** loop on all words in 'to' line...append to full_to as
	    we go along, until done or length > len.  Modified to
	    know that stuff in parens are comments...Returns non-zero
	    if it changed the information as it copied it across...
	**/

	register int	i, j, k, l,
			changed = 0, in_parens = 0,
			expanded_information = 0,
			eliminated = 0;
	int too_long = FALSE;
	char word[SLEN], next_word[SLEN], *ptr, buffer[SLEN];
	char new_to_list[VERY_LONG_STRING];
	char elim_list[SLEN], word_a[SLEN], next_word_a[SLEN];
	char *qstrpbrk(), *gecos;
	extern char *get_full_name(), *get_alias_address();

	new_to_list[0] = '\0';

	i = get_word(to, 0, word, sizeof(word));

	full_to[0] = '\0';

	elim_list[0] = '\0';

	/** Look for addresses to be eliminated from aliases **/
	while (i > 0) {

	  j = get_word(to, i, next_word, sizeof(next_word));

	  if(word[0] == '(')
	    in_parens++;

	  if (in_parens) {
	    if(word[strlen(word)-1] == ')')
	      in_parens--;
	  }

	  else if (word[0] == '-'){
	    for (k=0; word[k]; word[k] = word[k+1],k++);
	    if (elim_list[0] != '\0')
	      strfcat(elim_list, " ", sizeof elim_list);
	    strfcat(elim_list, word, sizeof elim_list);
	  }
	  if ((i = j) > 0)
	    strfcpy(word, next_word, sizeof word);
	}

	if (elim_list[0] != '\0')
	  eliminated++;

	i = get_word(to, 0, word, sizeof(word));

	while (i > 0) {

	  j = get_word(to, i, next_word, sizeof(next_word));

try_new_word:
	  if(word[0] == '(')
	    in_parens++;

	  if (in_parens) {
	    char c1 = '\0', c2 = '\0';
	    char *word1 = word;
#ifdef MIME
	    char temp[LONG_STRING];
#endif

	    if (word1[0] == '(') {
	      c1 = '(';
	      word1++;
	    }
	    if(word1[strlen(word1)-1] == ')') {
	      in_parens--;
	      c2 = ')';
	      word1[strlen(word1)-1] = '\0';
	    }
#ifdef MIME
	    if (!allow_no_hdrencoding) {
	      rfc1522_encode_text(temp,sizeof temp,word1,HDR_COMMENT);
	      word1 = temp;
	    } 
#endif 
	    strfcat(full_to, " ", size_full_to);
	    if (c1)
	      strfcat(full_to, "(", size_full_to);
	    strfcat(full_to, word1, size_full_to-1);
	    if (c2)
	      strfcat(full_to, ")", size_full_to);
	  }

	  else if (word[0] == '-') {
	  }

	  else if (qstrpbrk(word,"!@:") != NULL) {
	    elm_sfprintf(full_to, size_full_to,
			 FRM("%s%s%s"), full_to,
			 full_to[0] != '\0'? ", " : "", word);
	  }
	  else if ((ptr = get_alias_address(word, TRUE, &too_long)) != NULL) {

	    /** check aliases for addresses to be eliminated **/
	    if (eliminated) {
	      k = get_word(strip_commas(ptr), 0, word_a, sizeof(word_a));

	      while (k > 0) {
		l = get_word(ptr, k, next_word_a, sizeof(next_word_a));
		if (in_list(elim_list, word_a) == 0)
		  elm_sfprintf(full_to, size_full_to,
			       FRM("%s%s%s"), full_to,
			       full_to[0] != '\0' ? ", " : "", word_a);
		if ((k = l) > 0)
		  strfcpy(word_a, next_word_a, sizeof word_a);
	      }
	    } else
	      elm_sfprintf(full_to, size_full_to,
			   FRM("%s%s%s"), full_to, 
			   full_to[0] != '\0'? ", " : "", ptr);
	    expanded_information++;
	  }
	  else if (too_long) {
	 /*
	  *   We don't do any real work here.  But we need some
	  *   sort of test in this line of tests to make sure
	  *   that none of the other else's are tried if the 
	  *   alias expansion failed because it was too long.
	  */
	      dprint(2,(debugfile,"Overflowed alias expansion for %s\n", word));
	  }
	  else if (strlen(word) > 0) {
#ifdef MIME
	    char temp[LONG_STRING];
#endif
	    if (valid_name(word)) {
	      if (j > 0 && next_word[0] == '(')	/* already has full name */
		gecos = NULL;
	      else				/* needs a full name */
		gecos = get_full_name(word);
#ifdef MIME
	    if (!allow_no_hdrencoding && gecos) {
	      /* from is not longer mime encoded */

	      rfc1522_encode_text(temp,sizeof temp,gecos,HDR_COMMENT);
	      gecos = temp;
	    } 
#endif 

#if defined(USE_DOMAIN)
	      elm_sfprintf(full_to, size_full_to,
			   FRM("%s%s%s@%s%s%s%s"),
			   full_to,
			   (full_to[0] ? ", " : ""),
			   word,
			   hostfullname,
			   (gecos ? " (" : ""),
			   (gecos ? gecos : ""),
			   (gecos ? ")" : ""));
#else /* USE_DOMAIN */
	      elm_sfprintf(full_to, size_full_to,
			   FRM("%s%s%s%s%s%s"),
			   full_to,
			   (full_to[0] ? ", " : ""),
			   word,
			   (gecos ? " (" : ""),
			   (gecos ? gecos : ""),
			   (gecos ? ")" : ""));
#endif /* USE_DOMAIN */
	    }
	    else if (check_only) {
		if (! isatty(fileno(stdin)) ) {
		    /*
		     *	batch mode error!
		     */
		    Raw(OFF);
		    fprintf(stderr,
			    catgets(elm_msg_cat, ElmSet, ElmCannotExpandNoCR,
				    "Cannot expand alias '%s'!\n"),
			    word);
		    fprintf(stderr,
			    catgets(elm_msg_cat, ElmSet, ElmUseCheckalias,
				    "Use \"checkalias\" to find valid addresses!\n"));
		    dprint(1, (debugfile,
			       "Can't expand alias %s - bailing out of build_address\n", 
			       word));
		    leave(0);
		}
		else {
		    printf(catgets(elm_msg_cat, ElmSet, ElmAliasUnknown,
			"(alias \"%s\" is unknown)\n\r"), word);
		    changed++;
		}
	    }
	    else {
	      elm_sfprintf(full_to, size_full_to,
			   FRM("%s%s%s"),
			   full_to,
			   (full_to[0] ? ", " : ""),
			   word);
	    }
	  }

	  /* and this word to the new to list */
	  if(*new_to_list != '\0')
	    strfcat(new_to_list, " ", sizeof new_to_list);
	  strfcat(new_to_list, word, sizeof new_to_list);

	  if((i = j) > 0)
	    strfcpy(word, next_word, sizeof word);
	}

	/* if new to list is different from original, update original */
	if (changed)
	  strfcpy(to, new_to_list, size_to);

	return( expanded_information > 0 ? 1 : 0 );
}

int fix_arpa_address(address, size)
     char *address;
     int size;
{
	/** Given a pure ARPA address, try to make it reasonable.

	    This means that if you have something of the form a@b@b make 
            it a@b.  If you have something like a%b%c%b@x make it a%b@x...
	**/

	register int host_count = 0, i;
	char     hosts[MAX_HOPS][LONG_STRING];	/* array of machine names */
	char     *host, *addrptr;
	extern char *get_token();

	/*  break down into a list of machine names, checking as we go along */
	
	addrptr = (char *) address;

	while ((host = get_token(addrptr, "%@", 2)) != NULL) {
	  for (i = 0; i < host_count && ! equal(hosts[i], host); i++)
	      ;

	  if (i == host_count) {
	    strfcpy(hosts[host_count++], host, sizeof hosts[host_count]);
	    if (host_count == MAX_HOPS) {
	       dprint(2, (debugfile, 
           "Can't build return address - hit MAX_HOPS in fix_arpa_address\n"));
	       error(catgets(elm_msg_cat, ElmSet, ElmCantBuildRetAddr,
			"Can't build return address - hit MAX_HOPS limit!"));
	       return(1);
	    }
	  }
	  else 
	    host_count = i + 1;
	  addrptr = NULL;
	}

	/** rebuild the address.. **/

	address[0] = '\0';

	for (i = 0; i < host_count; i++)
	  elm_sfprintf(address, size,
		       FRM("%s%s%s"), address, 
		       address[0] == '\0'? "" : 
		       (i == host_count - 1 ? "@" : "%"),
		       hosts[i]);

	return 0;
}
